! ESA (C) 2000-2021
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

! DeltaC sensitivity routines based on code by Christian Motch (OAS)

module esens_deltac_mod

  use errorHandling
  use dal
  use cal
  use edetect_io_sens
  use coordinate    ! requires ssclib in DEPEND and ssc in src/Makefile
  use wcs_aux       ! -"-

  implicit none

  private

  public :: rpsf, deltac_diff, deltac_fit

  integer, public :: ecut, isize

  real(kind=single), dimension(:),       allocatable, public :: ecf
  real(kind=single), dimension(:),       allocatable, public :: energies
  real,              dimension(:,:,:,:), allocatable         :: psfima
  real(kind=single), dimension(:),       allocatable, public :: fluxed_stot
  integer,           dimension(:,:,:),   allocatable, public :: ipsf1
  integer,           dimension(:,:,:),   allocatable, public :: ipsf2
  real(kind=single), dimension(:,:,:),   allocatable, public :: apsf1
  real(kind=single), dimension(:,:,:),   allocatable, public :: apsf2

  real(kind=double), public :: dcd_comp      ! deltac_diff input
  integer,  public :: dcd_x, dcd_y, dcd_xmin, dcd_xmax, dcd_ymin, dcd_ymax


contains

!------------------------------------------------------------------------------
! Read binned CAL PSFs at a given position and calculate the theta
! interpolation at each image pixel

  !----------------------------------------------------------------------------
  subroutine rpsf(psf_id, doff, ntheta)
  !----------------------------------------------------------------------------

    integer, intent(in)        :: psf_id           !! model of the PSF shape
    real,    intent(in)        :: doff             !! offset step size
    integer, intent(out)       :: ntheta           !! number of off-axis angles

    real, dimension(:,:), pointer :: pima          !! CAL PSF image
    type (PsfT)                :: ppsf
    type(PsfBinSizeT)          :: binSize
    type(PsfValidityRangesT)   :: vrange
    type(PSFwindowT)           :: window

    integer                    :: psfnx, psfny     !! size of CAL PSF image
    integer                    :: astat            !! allocation status

    real(kind=double) :: E, sec2mmX, sec2mmY, pixX, pixY, pix2
    real(kind=double) :: theta, phi, xTan, yTan  !! coordinates
    real(kind=double) :: thetamax                !! maximum offset used

    integer :: iinst, iband, ipoint, itheta, ix,iy, imix,imiy  !! loop variables

!    real(kind=single), parameter :: maxpsfhsize = 400.  !! arcsec
    real(kind=double), parameter :: dpi=4d0*datan(1.d0), dpi180=dpi/180d0


    write(errstr, *) "Retrieving PSFs ..."
    call message(AppMsg, Verbose)

    ! preparatory stuff
    pixX = dabs(crdelt1)*3600d0
    pixY = dabs(crdelt2)*3600d0
    pix2 = pixX * pixY

    ! get the PSF validity range from pn information (theta independent of inst)
    ! [contains vrange%thetaMin, vrange%phiMin, vrange%eMin and ...Max]
    call CAL_setState1(instrument=EPN, AccuracyLevel=psf_id)
    vrange = CAL_PsfValidityRanges()

    ! number of offaxis steps
    ntheta = int(vrange%thetaMax/doff)+1
    ! resulting maximum off-axis angle for which PSFs are read
    thetamax = doff*(ntheta-1)

    ! allocate result arrays
    allocate(psfima(isize,isize,nim,ntheta), stat=astat)
                               if (astat.ne.0) call error("notEnoughMemory", &
                               "Not enough memory to allocate array 'psfima'")
    psfima = 0.
    allocate(ipsf1(imusex,imusey,nim), stat=astat)
                               if (astat.ne.0) call error("notEnoughMemory", &
                               "Not enough memory to allocate array 'ipsf1'")
    ipsf1 = 0
    allocate(ipsf2(imusex,imusey,nim), stat=astat)
                               if (astat.ne.0) call error("notEnoughMemory", &
                               "Not enough memory to allocate array 'ipsf2'")
    ipsf2 = 0
    allocate(apsf1(imusex,imusey,nim), stat=astat)
                               if (astat.ne.0) call error("notEnoughMemory", &
                               "Not enough memory to allocate array 'apsf1'")
    apsf1 = 0.
    allocate(apsf2(imusex,imusey,nim), stat=astat)
                               if (astat.ne.0) call error("notEnoughMemory", &
                               "Not enough memory to allocate array 'apsf2'")
    apsf2 = 0.
    allocate(fluxed_stot(ntheta), stat=astat)
                               if (astat.ne.0) call error("notEnoughMemory", &
                            "Not enough memory to allocate array 'fluxed_stot'")
    fluxed_stot = 0.

    ! loop over instruments
    do iinst = 1,num_instr

       ! set instrument
       select case (expip(iinst))
       case (2)                         !!! MOS 1
          call CAL_setState1(instrument=EMOS1, AccuracyLevel=psf_id)
       case (3)                         !!! MOS 2
          call CAL_setState1(instrument=EMOS2, AccuracyLevel=psf_id)
       case default ! (1)               !!! PN (& unknown)
          call CAL_setState1(instrument=EPN,   AccuracyLevel=psf_id)
       end select

       ! convert pixel size to arcsec to mm
       sec2mmX = 1. &
                 / CAL_getMiscellaneousDataValue('LINCOORD::LINCOORD%ARC_MM_X')
       sec2mmY = 1. &
                 / CAL_getMiscellaneousDataValue('LINCOORD::LINCOORD%ARC_MM_Y')

       ! keep phi fixed at zero (rotation symmetry)
       phi = 0.0

       ! loop over energy bands
       do iband = 1,num_image(iinst)

          ipoint=ip(iinst, iband)

          ! set mid-band energy (task parameter)
          E = energies(ipoint)

          ! loop over off-axis angles theta
          ! (theta = (itheta-1)*doff , phi constant)
          do itheta = 1,ntheta

             ! get PSF from CAL
             ! directly in image-pixel bin size; can remain centered since the
             ! PSFs are interpolated anyway (-> maximum at pixel (ecut,ecut) )
             ppsf = Cal_getPSF(E, dble(itheta-1)*doff, phi)
             ! set PSF bin size
             binSize%binX = pixX * sec2mmX
             binSize%binY = pixY * sec2mmY
             ! set PSF window size
             window%halfX = ecut * binSize%binX   ! maxpsfhsize*sec2mmX
             window%halfY = ecut * binSize%binY   ! maxpsfhsize*sec2mmY
             call CAL_psfSetBinSize(ppsf, binSize)
             call CAL_psfSetPosAngle(ppsf, pa_pnt(iinst)*dpi180)
             call CAL_psfImage (ppsf, window, pima)
             psfnx = size(pima,1)/2
             psfny = size(pima,2)/2
             ! check sizes
             if (psfnx.lt.ecut .or. psfny.lt.ecut) &
                  call error("SizeMismatch", "PSF array does not have the &
                                             &requested size. Exiting.")

             ! normalize PSF and copy it to result array of width isize
             ! -> CAL PSFs are normalized as 1 = pixX*pixY*sum_i(PSF_i)
             psfima(:,:,ipoint,itheta) = &
                  pima(psfnx-ecut:psfnx+ecut, psfny-ecut:psfny+ecut) * pix2

             call CAL_releaseMemory(pima)

             ! off-axis dependent PSF flux correction
             ! (actually independent of the camaras used)
             ! and beware of rounding effects
             fluxed_stot(itheta) = fluxed_stot(itheta) &
                  + ecf(ipoint) * min(sum(psfima(:,:,ipoint,itheta)), 1d0)

          enddo   ! loop over off-axis angles

       enddo   ! loop over energy bands

       ! PSF offset map: For each image position and for each telescope, store
       ! the two closest psf angles and the linear combination between them
       ! ipsf1(ix,iy,iinst), ipsf2(ix,iy,iinst),
       ! apsf1(ix,iy,iinst), apsf2(ix,iy,iinst)
       ! Final psf is (apsf1*ipsf1 + apsf2*ipsf2)/(apsf1+apsf2)
       iy = imymin
       do imiy = 1,imusey
          ix = imxmin
          do imix = 1,imusex
             ! if a valid position (same mask for all bands)
             if (mask_image(ix,iy,ip(iinst,1))) then

                ! get the actual offaxis angle at pixel center
                call pixelsToTan(real(ix)+.5, real(iy)+.5, wcs, xTan, yTan)
                call tanToTel( &
                     xTan, yTan, &
                     wcs%x%refWorld * dpi180, wcs%y%refWorld * dpi180, &
                     attitudes(ip(iinst,1)), timestamps(ip(iinst,1)), &
                     theta, phi)
                ! limit to range of read PSFs
                if (theta.gt.thetamax) theta = thetamax

                ! linear combination parameters
                ipsf1(imix,imiy,iinst) = int(theta/doff) + 1
                ipsf2(imix,imiy,iinst) = min(ipsf1(imix,imiy,iinst) + 1, ntheta)
                apsf2(imix,imiy,iinst) = (theta &
                                       - doff*(ipsf1(imix,imiy,iinst)-1))/doff
                apsf1(imix,imiy,iinst) = 1.0 - apsf2(imix,imiy,iinst)

             endif  ! end if valid position
             ix = ix+1
          enddo  ! end loop over x coordinate
          iy = iy+1
       enddo  ! end loop over y coordinate

    enddo   ! loop over instruments

  end subroutine rpsf




!------------------------------------------------------------------------------
! Return the summed DeltaC over all bands  (CM)
! Accepts only one argument, so it can be included in zeroot. Other variables
! are provided via a public statement and have to be set beforehand by the
! calling subroutine.

  !----------------------------------------------------------------------------
  real (kind=8) function deltac_diff(norrate)
  !----------------------------------------------------------------------------

    real(kind=double) :: norrate
    real(kind=single) :: deltac, cbg, cbgsx, cband, cbandpsf, clog, csum
    real(kind=single) :: rpsf1, rpsf2, sum1, sum2
    integer           :: ixima, iyima, ixmin
    integer           :: ix,iy, npsf1,npsf2, iinst, iband
    integer           :: isize, ipoint

    ! preparatory stuff
    deltac = 0.

     ! loop over instruments
    do iinst = 1,num_instr

       ! if a valid position (same mask for all bands)
       if (mask_image(dcd_x,dcd_y,ip(iinst,1))) then

          ! PSF interpolation factors
          npsf1 = ipsf1(dcd_x-imxmin+1,dcd_y-imymin+1,iinst)
          npsf2 = ipsf2(dcd_x-imxmin+1,dcd_y-imymin+1,iinst)
          rpsf1 = apsf1(dcd_x-imxmin+1,dcd_y-imymin+1,iinst)
          rpsf2 = apsf2(dcd_x-imxmin+1,dcd_y-imymin+1,iinst)

          ! loop over energy bands
          do iband = 1,num_image(iinst)

             ipoint=ip(iinst, iband)

             cbg = 0.0
             cbgsx = 0.0
             sum1 = 0.0
             sum2 = 0.0
             ! full source cnts in band n
             cband = exp_image(dcd_x,dcd_y,ipoint) * norrate * ecf(ipoint)

             ! loop over PSF coordinates, limited to exposed area
             ! (the limit achecks allow the calculate the sensitivity over the
             ! whole image area if exposed)
             ixmin = dcd_x - ecut + dcd_xmin     ! image coordinate
             iyima = dcd_y - ecut + dcd_ymin     ! image coordinate
             do iy = dcd_ymin, dcd_ymax          ! PSF coordinate
                ixima = ixmin                    ! image coordinate
                do ix = dcd_xmin, dcd_xmax       ! PSF coordinate
                   ! bg_image available in valid pixels only
                   if (mask_image(ixima,iyima,ipoint)) then
                      cbandpsf =  cband*( rpsf1*psfima(ix,iy,ipoint,npsf1) &
                                        + rpsf2*psfima(ix,iy,ipoint,npsf2) )
                      csum = cbandpsf + bg_image(ixima,iyima,ipoint)
                      clog = log(csum)
                      cbg = cbg - log(bg_image(ixima,iyima,ipoint)) * csum
                      sum1 = sum1 + clog * cbandpsf
                      sum2 = sum2 + clog * bg_image(ixima,iyima,ipoint)
                   endif
                   ixima = ixima + 1
                enddo
                iyima = iyima + 1
             enddo    ! end loop over PSF coordinates

             cbgsx = cbg - cband +sum1+sum2
             deltac = deltac + 2.*cbgsx

          enddo   ! end loop over energy bands
       endif   ! end if valid position
    enddo   ! end loop over instruments

    deltac_diff = deltac - dcd_comp

  end function deltac_diff




!------------------------------------------------------------------------------
! Pixel-wise sensitivity fit, calling deltac_diff and zeroot

  !----------------------------------------------------------------------------
  real(kind=double) function deltac_fit(dcomp, ix, iy, nbands)
  !----------------------------------------------------------------------------

  integer, intent(in)           :: nbands             !! number of energy bands
  integer, intent(in)           :: ix,iy              !! deltac_diff input
  real(kind=double), intent(in) :: dcomp              !! deltac_diff input
  real(kind=double)             :: b, machep, t       !! zeroot input
  integer     ::     niter, nitermax       !! limit the number of fit iterations

  nitermax = 10000

  !! call deltac_diff
  ! variables have to be set here to limit deltac_diff to one argument
  b = 1d-2
  dcd_comp = dcomp
  dcd_x = ix
  dcd_y = iy
  ! PSF range within image area (used by deltac_diff)
  ! (ixima>=1 -> dcd_x-ecut+ix>=1 -> ix>=1+ecut-dcd_x  etc)
  dcd_xmin = max(  1,        1+ecut-dcd_x)
  dcd_xmax = min(isize,npix(1)+ecut-dcd_x)
  dcd_ymin = max(  1,        1+ecut-dcd_y)
  dcd_ymax = min(isize,npix(2)+ecut-dcd_y)

  niter = 0
  do while(deltac_diff(b).lt.0. .and. niter.lt.nitermax)
     b = 5. * b
     niter = niter + 1
  enddo
  if (niter.ge.nitermax) &
       call warning("noConvergence", "DeltaC fit ceased without convergence")

  !! call zeroot
  t = 1d-6
  machep = 1d-15
  deltac_fit = zeroot(0d0, b, machep, t, deltac_diff, niter, nitermax)
  if (niter.ge.nitermax) &
       call warning("noConvergence", "minimum fit ceased without convergence")

  end function deltac_fit



!------------------------------------------------------------------------------
!------------------------------------------------------------------------------
!! Modifications IT:
!! * rename to avoid confusion with the generic zero
!! * introduce a maximum number of iterations to catch non-converging fits

function zeroot ( a, b, machep, t, f, iter, imax )

!*******************************************************************************
!
!! ZERO seeks the root of a function F(X) in an interval [A,B].
!
!  Discussion:
!
!    The interval [A,B] must be a change of sign interval for F.
!    That is, F(A) and F(B) must be of opposite signs.  Then
!    assuming that F is continuous implies the existence of at least
!    one value C between A and B for which F(C) = 0.
!
!    The location of the zero is determined to within an accuracy
!    of 6 * MACHEPS * abs ( C ) + 2 * T.
!
!    Thanks to Thomas Secretin for pointing out a transcription error in the
!    setting of the value of P, 11 February 2013.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    11 February 2013
!
!  Author:
!
!    Original FORTRAN77 version by Richard Brent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Richard Brent,
!    Algorithms for Minimization Without Derivatives,
!    Dover, 2002,
!    ISBN: 0-486-41998-3,
!    LC: QA402.5.B74.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) A, B, the endpoints of the change of
!    sign interval.
!
!    Input, real ( kind = 8 ) MACHEP, an estimate for the relative machine
!    precision.
!
!    Input, real ( kind = 8 ) T, a positive error tolerance.
!
!    Input, external real ( kind = 8 ) F, the name of a user-supplied
!    function, of the form "FUNCTION F ( X )", which evaluates the
!    function whose zero is being sought.
!
!    Output, real ( kind = 8 ) ZERO, the estimated value of a zero of
!    the function F.
!
  implicit none

  real ( kind = 8 ) a
  real ( kind = 8 ) b
  real ( kind = 8 ) c
  real ( kind = 8 ) d
  real ( kind = 8 ) e
  real ( kind = 8 ) f
  real ( kind = 8 ) fa
  real ( kind = 8 ) fb
  real ( kind = 8 ) fc
  real ( kind = 8 ) m
  real ( kind = 8 ) machep
  real ( kind = 8 ) p
  real ( kind = 8 ) q
  real ( kind = 8 ) r
  real ( kind = 8 ) r1
  real ( kind = 8 ) s
  real ( kind = 8 ) sa
  real ( kind = 8 ) sb
  real ( kind = 8 ) t
  real ( kind = 8 ) tol
  real ( kind = 8 ) zeroot
  integer iter
  integer imax
!
!  Make local copies of A and B.
!
  sa = a
  sb = b
  fa = f ( sa )
  fb = f ( sb )

  c = sa
  fc = fa
  e = sb - sa
  d = e

!!!  do
  iter = 0
  do while (iter < imax)

    if ( abs ( fc ) < abs ( fb ) ) then

      sa = sb
      sb = c
      c = sa
      fa = fb
      fb = fc
      fc = fa

    end if

    tol = 2.0D+00 * machep * abs ( sb ) + t
    m = 0.5D+00 * ( c - sb )

    if ( abs ( m ) <= tol .or. fb == 0.0D+00 ) then
      exit
    end if

    if ( abs ( e ) < tol .or. abs ( fa ) <= abs ( fb ) ) then

      e = m
      d = e

    else

      s = fb / fa

      if ( sa == c ) then

        p = 2.0D+00 * m * s
        q = 1.0D+00 - s

      else

        q = fa / fc
        r = fb / fc
        p = s * ( 2.0D+00 * m * q * ( q - r ) - ( sb - sa ) * ( r - 1.0D+00 ) )
        q = ( q - 1.0D+00 ) * ( r - 1.0D+00 ) * ( s - 1.0D+00 )

      end if

      if ( 0.0D+00 < p ) then
        q = - q
      else
        p = - p
      end if

      s = e
      e = d

      if ( 2.0D+00 * p < 3.0D+00 * m * q - abs ( tol * q ) .and. &
        p < abs ( 0.5D+00 * s * q ) ) then
        d = p / q
      else
        e = m
        d = e
      end if

    end if

    sa = sb
    fa = fb

    if ( tol < abs ( d ) ) then
      sb = sb + d
    else if ( 0.0D+00 < m ) then
      sb = sb + tol
    else
      sb = sb - tol
    end if

    fb = f ( sb )

    if ( ( 0.0D+00 < fb .and. 0.0D+00 < fc ) .or. &
         ( fb <= 0.0D+00 .and. fc <= 0.0D+00 ) ) then
      c = sa
      fc = fa
      e = sb - sa
      d = e
    end if

    iter = iter+1

  end do

  zeroot = sb

  return
end function zeroot

!------------------------------------------------------------------------------

end module esens_deltac_mod
